@@ -19,6 +19,8 @@
 
 #include <glib-object.h>
 #include <hildon/hildon-program.h>
+#include <hildon/hildon-file-chooser-dialog.h>
+#include <hildon-mime.h>
 #include <libosso.h>
 #include <osso-log.h> /* OSSO Debug macros */
 #include <rtcom-eventlogger/eventlogger.h>
@@ -33,6 +35,7 @@
 #include "tp-status-feed-private.h"
 #include "timeline-view.h"
 #include "contact-status-view.h"
+#include "contact-history-view.h"
 #include "delete-status-window.h"
 #include "eventlogger-util.h"
 
@@ -52,6 +55,21 @@ static void show_my_status (GtkWidget *widget, gpointer data);
 
 /************* main status menu **************/
 
+static void
+_friends_view_deleted_cb (DeleteStatusWindow *del_window,
+                          gpointer data)
+{
+  TpStatusFeedPrivate *priv = TP_STATUS_FEED_GET_PRIVATE (data);
+
+  /* refresh friends view */
+  timeline_view_refresh (
+      TIMELINE_VIEW (priv->main_status.friend_view));
+
+  /* disconnect the "deleted" signal */
+  g_signal_handlers_disconnect_by_func (del_window,
+      _friends_view_deleted_cb, data);
+}
+
 static void
 menu_delete_cb (GtkWidget *widget,
                 gpointer data)
@@ -64,9 +82,17 @@ menu_delete_cb (GtkWidget *widget,
   GtkWidget *del_window = g_object_ref_sink (
       delete_status_window_new (priv->aggregator));
 
-  if (priv->friends_flag)
+  if (priv->friends_flag) {
     delete_status_window_populate_friends_status (
         DELETE_STATUS_WINDOW (del_window));
+
+    /* Only need to refresh friends view after deleting statuses
+     * when it's grouped by contact */
+    if (priv->main_status.group_by == RTCOM_EL_QUERY_GROUP_BY_CONTACT) {
+      g_signal_connect (DELETE_STATUS_WINDOW (del_window), "deleted",
+          G_CALLBACK (_friends_view_deleted_cb), statusfeed);
+    }
+  }
   else
     delete_status_window_populate_my_status (
             DELETE_STATUS_WINDOW (del_window));
@@ -94,6 +120,107 @@ menu_accounts_cb (GtkWidget *widget,
                           priv->main_status.window, TRUE);
 }
 
+static void
+backup_dialog_response_cb (GtkDialog *dialog,
+                           gint response,
+                           gpointer data)
+{
+  if (response == GTK_RESPONSE_OK) {
+    DBusConnection *connection;
+    gchar *filename;
+    gchar *dir;
+
+    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+    dir = g_build_filename (g_get_home_dir (), STORAGE_DIR, NULL);
+
+    if (g_str_has_prefix (filename, dir)) {
+
+      /* Get system bus */
+      connection = dbus_bus_get (DBUS_BUS_SESSION, NULL);
+
+      if (connection) {
+        hildon_mime_open_file (connection, filename);
+        dbus_connection_unref (connection);
+      }
+      else {
+        hildon_banner_show_information (
+            GTK_WIDGET (dialog),
+            NULL, "File open failed");
+      }
+    }
+    else {
+      hildon_banner_show_information (
+          GTK_WIDGET (dialog),
+          NULL, "Wrong directory");
+    }
+
+    g_free (filename);
+    g_free (dir);
+  }
+}
+
+static void
+menu_backup_cb (GtkWidget *widget,
+                gpointer data)
+{
+  TpStatusFeedPrivate *priv = TP_STATUS_FEED_GET_PRIVATE (data);
+  GtkWidget *dialog;
+  gchar *dir;
+
+  dir = g_build_filename (g_get_home_dir (),
+      STORAGE_DIR, NULL);
+
+  if (g_file_test (dir, G_FILE_TEST_EXISTS) == FALSE) {
+    hildon_banner_show_information (
+        GTK_WIDGET (priv->main_status.window),
+        NULL, "Statuses will be backed up after you close FriendStatus program");
+    goto END;
+  }
+
+  dialog = hildon_file_chooser_dialog_new (
+      GTK_WINDOW (priv->main_status.window),
+      GTK_FILE_CHOOSER_ACTION_OPEN);
+
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), dir);
+
+  g_signal_connect (dialog, "response",
+      G_CALLBACK (backup_dialog_response_cb), data);
+
+  g_signal_connect (dialog, "destroy",
+      G_CALLBACK (gtk_widget_destroy), data);
+
+  gtk_widget_show_all (GTK_WIDGET (dialog));
+
+END:
+  g_free (dir);
+}
+
+static void
+menu_about_cb (GtkWidget *widget,
+               gpointer data)
+{
+  gchar *authors [] = {AUTHOR, NULL};
+
+  GtkWidget *about_dialog = g_object_new (GTK_TYPE_ABOUT_DIALOG,
+      "program-name", PROGRAM_NAME,
+      "logo-icon-name", LOGO_ICON_NAME,
+      "authors", authors,
+      "comments", COMMENTS,
+      "website", WEBSITE,
+      NULL);
+
+  gtk_window_set_destroy_with_parent (GTK_WINDOW (about_dialog), TRUE);
+
+  g_signal_connect (about_dialog, "response",
+      G_CALLBACK (gtk_widget_destroy), about_dialog);
+
+  g_signal_connect (about_dialog, "destroy",
+      G_CALLBACK (gtk_widget_destroy), about_dialog);
+
+  gtk_window_set_modal (GTK_WINDOW (about_dialog), TRUE);
+  gtk_widget_show (about_dialog);
+}
+
 static inline GtkWidget *
 menu_append_item (GtkWidget  *menu,
                   const char *label,
@@ -164,7 +291,13 @@ main_status_window_menu (TpStatusFeed *statusfeed)
         menu, "Settings", TRUE, G_CALLBACK (menu_settings_cb), statusfeed);
 #endif
     priv->main_menu.accounts = menu_append_item (
-        menu, "My accounts", TRUE, G_CALLBACK (menu_accounts_cb), statusfeed);
+        menu, "Accounts", TRUE, G_CALLBACK (menu_accounts_cb), statusfeed);
+
+    priv->main_menu.backup = menu_append_item (
+        menu, "View statuses backup", TRUE, G_CALLBACK (menu_backup_cb), statusfeed);
+
+    priv->main_menu.about = menu_append_item (
+        menu, "About", TRUE, G_CALLBACK (menu_about_cb), statusfeed);
   }
 
   g_object_ref_sink (menu);
@@ -175,6 +308,25 @@ main_status_window_menu (TpStatusFeed *statusfeed)
 
 /************* contact status menu **************/
 
+static void
+_contact_status_deleted_cb (DeleteStatusWindow *del_window,
+                            gpointer data)
+{
+  TpStatusFeedPrivate *priv = TP_STATUS_FEED_GET_PRIVATE (data);
+
+  /* refresh contact_status_view */
+  contact_status_view_refresh (
+      CONTACT_STATUS_VIEW (priv->contact_status.view));
+
+  /* refresh main view */
+  timeline_view_refresh (
+      TIMELINE_VIEW (priv->main_status.friend_view));
+
+  /* disconnect the "deleted" signal */
+  g_signal_handlers_disconnect_by_func (del_window,
+      _contact_status_deleted_cb, data);
+}
+
 static void
 contact_menu_delete_cb (GtkWidget *widget,
                         gpointer data)
@@ -185,6 +337,9 @@ contact_menu_delete_cb (GtkWidget *widget,
   GtkWidget *del_window = g_object_ref_sink (
       delete_status_window_new (priv->aggregator));
 
+  g_signal_connect (DELETE_STATUS_WINDOW (del_window), "deleted",
+      G_CALLBACK (_contact_status_deleted_cb), statusfeed);
+
   const gchar *account = contact_status_view_get_contact_account (
       CONTACT_STATUS_VIEW (priv->contact_status.view));
   const gchar *im_field = contact_status_view_get_contact_im_field (
@@ -344,6 +499,37 @@ create_contact_status_window (TpStatusFeed *statusfeed)
 
 /******** Main view: main status window ********/
 
+static gboolean
+tp_status_feed_cleanup (gpointer data)
+{
+  TpStatusFeedPrivate *priv = TP_STATUS_FEED_GET_PRIVATE (data);
+
+  eventlogger_util_save_statuses_to_storage (priv->eventlogger,
+      priv->last_friend_status_id, priv->last_my_status_id);
+
+  /* TODO delete too old statuses from el.db */
+
+  gtk_main_quit ();
+
+  return FALSE;
+}
+
+static gboolean
+tp_status_feed_quit (GtkWidget *window,
+                     GdkEvent *event,
+                     gpointer data)
+{
+  gtk_widget_hide_all (window);
+
+  /* TODO first save statuses to file storage
+   * then clean up el.db?
+   * The process might be long, so we should hide UI for now
+   * and do other tasks in an idle function. Once done, quit the main */
+  g_idle_add ((GSourceFunc) tp_status_feed_cleanup, data);
+
+  return TRUE;
+}
+
 static void
 update_status_cb (GtkWidget *widget,
                   gpointer data)
@@ -538,6 +724,35 @@ select_group_cb (GtkWidget *widget,
   gtk_button_set_image_position (GTK_BUTTON (widget), GTK_POS_LEFT);
 }
 
+static void
+me_view_refresh_history (TpStatusFeedPrivate *priv)
+{
+  const gchar *local_uid; /* osso-abook-self */
+
+  if (priv->main_status.me_history_view) {
+    /* TODO Disconnect the "row-activated" signal */
+    gtk_container_remove (GTK_CONTAINER (priv->main_status.me_view_vbox),
+        priv->main_status.me_history_view);
+  }
+
+  priv->main_status.me_history_view = contact_history_view_new ();
+  gtk_container_add (GTK_CONTAINER (priv->main_status.me_view_vbox),
+      priv->main_status.me_history_view);
+
+  /* remote_uid = local_uid, both must not be NULL */
+  local_uid = osso_abook_contact_get_uid (
+      OSSO_ABOOK_CONTACT (priv->self_contact));
+
+  contact_history_view_set_eventlogger (
+      CONTACT_HISTORY_VIEW (priv->main_status.me_history_view),
+      priv->eventlogger);
+  contact_history_view_update_contact (
+      CONTACT_HISTORY_VIEW (priv->main_status.me_history_view),
+      local_uid, local_uid);
+
+  /* TODO g_signal_connect */
+}
+
 static void
 show_my_status (GtkWidget *widget,
                 gpointer data)
@@ -548,13 +763,13 @@ show_my_status (GtkWidget *widget,
   if (FALSE == priv->friends_flag)
     return ;
 
+  me_view_refresh_history (priv);
+
   gtk_window_set_title (GTK_WINDOW (priv->main_status.window),
                         "My Status");
-  gtk_widget_show (priv->main_status.me_view);
+  gtk_widget_show_all (priv->main_status.me_view);
   gtk_widget_hide (priv->main_status.friend_view);
   priv->friends_flag = FALSE;
-  timeline_view_populate_my_status (TIMELINE_VIEW (
-                                    priv->main_status.me_view));
 }
 
 static void
@@ -599,6 +814,68 @@ main_status_button_new (const char *icon_name,
   return button;
 }
 
+static gboolean
+startup_populate_friend_view (gpointer data)
+{
+  timeline_view_populate_all_status (data);
+
+  return FALSE;
+}
+
+static GtkWidget *
+create_me_view (TpStatusFeed *statusfeed)
+{
+  TpStatusFeedPrivate *priv = TP_STATUS_FEED_GET_PRIVATE (statusfeed);
+  OssoABookSelfContact *self_contact;
+  GtkWidget *me_view;
+  GtkWidget *panarea;
+  GtkWidget *hbox;
+  GtkWidget *image;
+  GtkWidget *bubble;
+
+  me_view = gtk_alignment_new (0, 0, 1, 1);
+  g_object_set (me_view,
+                "top-padding", HILDON_MARGIN_HALF,
+                "left-padding", HILDON_MARGIN_DOUBLE,
+                "right-padding", HILDON_MARGIN_DOUBLE,
+                NULL);
+
+  panarea = hildon_pannable_area_new ();
+  g_object_set (panarea,
+                "mov-mode", HILDON_MOVEMENT_MODE_VERT,
+                "hscrollbar-policy", GTK_POLICY_NEVER,
+                NULL);
+
+  priv->main_status.me_view_vbox = gtk_vbox_new (FALSE, HILDON_MARGIN_DEFAULT);
+
+  hbox = gtk_hbox_new (FALSE, 0);
+  self_contact = osso_abook_self_contact_get_default ();
+  image = osso_abook_avatar_image_new_with_avatar (
+      OSSO_ABOOK_AVATAR (self_contact), OSSO_ABOOK_PIXEL_SIZE_AVATAR_CHOOSER);
+  priv->main_status.update_button = hildon_gtk_button_new (HILDON_SIZE_AUTO);
+  gtk_button_set_relief (
+      GTK_BUTTON (priv->main_status.update_button), GTK_RELIEF_NONE);
+  bubble = gtk_image_new_from_icon_name (
+      "status_logger_bubble", HILDON_ICON_SIZE_FINGER);
+  gtk_container_add (
+      GTK_CONTAINER (priv->main_status.update_button), bubble);
+
+  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), priv->main_status.update_button, FALSE, FALSE, 0);
+
+  gtk_box_pack_start (GTK_BOX (priv->main_status.me_view_vbox), hbox, FALSE, FALSE, 0);
+  hildon_pannable_area_add_with_viewport (HILDON_PANNABLE_AREA (panarea),
+      priv->main_status.me_view_vbox);
+  gtk_container_add (GTK_CONTAINER (me_view), panarea);
+
+  g_signal_connect (G_OBJECT (priv->main_status.update_button),
+                    "clicked",
+                    G_CALLBACK (update_status_cb),
+                    statusfeed);
+
+  return me_view;
+}
+
 /**
  * create_main_status_window:
  *
@@ -612,9 +889,6 @@ create_main_status_window (TpStatusFeed *statusfeed)
   GtkWidget *window;
   GtkWidget *top_box, *hbox;
   HildonAppMenu *menu;
-  OssoABookSelfContact *self_contact;
-  GtkWidget *image;
-  GtkWidget *bubble;
   TpStatusFeedPrivate *priv = TP_STATUS_FEED_GET_PRIVATE (statusfeed);
 
   window = hildon_stackable_window_new ();
@@ -638,17 +912,7 @@ create_main_status_window (TpStatusFeed *statusfeed)
       HILDON_SIZE_FINGER_HEIGHT);
 
   /* Me View */
-  priv->main_status.me_view = g_object_ref_sink (timeline_view_new (TRUE));
-  self_contact = osso_abook_self_contact_get_default ();
-  image = osso_abook_avatar_image_new_with_avatar (
-      OSSO_ABOOK_AVATAR (self_contact), OSSO_ABOOK_PIXEL_SIZE_AVATAR_CHOOSER);
-  priv->main_status.update_button = hildon_gtk_button_new (HILDON_SIZE_AUTO);
-  gtk_button_set_relief (
-      GTK_BUTTON (priv->main_status.update_button), GTK_RELIEF_NONE);
-  bubble = gtk_image_new_from_icon_name (
-      "status_logger_bubble", HILDON_ICON_SIZE_FINGER);
-  gtk_container_add (
-      GTK_CONTAINER (priv->main_status.update_button), bubble);
+  priv->main_status.me_view = create_me_view (statusfeed);
 
   /* Packing */
   top_box = timeline_view_get_action_area_box (
@@ -656,11 +920,6 @@ create_main_status_window (TpStatusFeed *statusfeed)
   gtk_container_add (GTK_CONTAINER (top_box), priv->main_status.group_button);
   gtk_container_add (GTK_CONTAINER (top_box), priv->main_status.abook_button);
 
-  top_box = timeline_view_get_top_box (
-        TIMELINE_VIEW (priv->main_status.me_view));
-  gtk_box_pack_start (GTK_BOX (top_box), image, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (top_box), priv->main_status.update_button, FALSE, FALSE, 0);
-
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), priv->main_status.friend_view, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), priv->main_status.me_view, TRUE, TRUE, 0);
@@ -685,14 +944,11 @@ create_main_status_window (TpStatusFeed *statusfeed)
                     G_CALLBACK (select_group_cb),
                     statusfeed);
 
-  g_signal_connect (G_OBJECT (priv->main_status.update_button),
-                    "clicked",
-                    G_CALLBACK (update_status_cb),
+  g_signal_connect (G_OBJECT(window),
+                    "delete_event",
+                    G_CALLBACK (tp_status_feed_quit),
                     statusfeed);
 
-  g_signal_connect (G_OBJECT(window), "delete_event",
-          G_CALLBACK(gtk_main_quit), NULL);
-
   /* By default collapsing all statuses by contact */
   priv->main_status.group_by = RTCOM_EL_QUERY_GROUP_BY_CONTACT;
   timeline_view_set_group_by (
@@ -706,17 +962,13 @@ create_main_status_window (TpStatusFeed *statusfeed)
    * TODO however, due to limitation of log_model, Friends status view can't
    * avoid to show my status update, and vice versa... unless log_model
    * will also support filtering by EventType */
-  timeline_view_populate_all_status (
-      TIMELINE_VIEW (priv->main_status.friend_view));
-/* TODO not set filter to me_view in order to improve startup performance
- *   timeline_view_populate_all_status (
-      TIMELINE_VIEW (priv->main_status.me_view)); */
+  g_idle_add ((GSourceFunc)startup_populate_friend_view,
+      priv->main_status.friend_view);
 
   gtk_widget_show_all (window);
   timeline_view_hide_search_bar (TIMELINE_VIEW (
                                  priv->main_status.friend_view));
-  timeline_view_hide_search_bar (TIMELINE_VIEW (
-                                 priv->main_status.me_view));
+
   return window;
 }
 
@@ -760,6 +1012,14 @@ tp_status_feed_init (TpStatusFeed *statusfeed)
   priv->eventlogger = timeline_view_get_eventlogger (
       TIMELINE_VIEW (priv->main_status.friend_view));
 
+  /* Get the last status id before logging any new statuses
+   * Saving statuses since this id to storage file
+   * when we later close the app */
+  priv->last_friend_status_id = eventlogger_util_get_last_status_id (
+      priv->eventlogger, "RTCOM_EL_EVENTTYPE_STATUS_FRIEND");
+  priv->last_my_status_id = eventlogger_util_get_last_status_id (
+      priv->eventlogger, "RTCOM_EL_EVENTTYPE_STATUS_MY");
+
   contact_status_view_set_eventlogger (CONTACT_STATUS_VIEW (
                                        priv->contact_status.view),
                                        priv->eventlogger);
@@ -785,11 +1045,11 @@ tp_status_feed_init (TpStatusFeed *statusfeed)
 
 static void
 tp_status_feed_dispose (GObject *obj)
-{/*
+{
   TpStatusFeed *statusfeed = TP_STATUS_FEED (obj);
   TpStatusFeedPrivate *priv = TP_STATUS_FEED_GET_PRIVATE (statusfeed);
 
-  osso_abook_roster_stop (OSSO_ABOOK_ROSTER (priv->aggregator));*/
+  osso_abook_roster_stop (OSSO_ABOOK_ROSTER (priv->aggregator));
 }
 
 static void
